@@ -27,6 +27,7 @@
 #include <rtcom-eventlogger/eventlogger.h>
 #include <sqlite3.h>
 #include <string.h>
+#include <gconf/gconf-client.h>
 #include <libosso-abook/osso-abook-init.h>
 #include <libosso-abook/osso-abook-aggregator.h>
 #include <libosso-abook/osso-abook-contact.h>
@@ -143,6 +144,10 @@ struct _ELHomeAppletPrivate
         gchar *local_id;
         gchar *group_uid;
         OssoABookContact *contact;
+
+        gboolean time_fmt_24h;
+
+        guint init_timer;
 };
 
 HD_DEFINE_PLUGIN_MODULE (ELHomeApplet, el_home_applet, HD_TYPE_HOME_PLUGIN_ITEM);
@@ -151,8 +156,8 @@ const gchar* g_module_check_init (GModule *module);
 const gchar*
 g_module_check_init (GModule *module)
 {
-	g_module_make_resident (module);
-	return NULL;
+        g_module_make_resident (module);
+        return NULL;
 }
 
 static void
@@ -645,6 +650,8 @@ show_contact (ELHomeApplet *self)
 {
         ELHomeAppletPrivate *priv = self->priv;
 
+        g_return_if_fail (priv->contact);
+
         gtk_label_set_text (GTK_LABEL (priv->sender),
                             osso_abook_contact_get_display_name (priv->contact));
         resize_sender (priv);
@@ -825,6 +832,11 @@ dispose (GObject *self)
 {
         ELHomeAppletPrivate *priv = EL_HOME_APPLET(self)->priv;
 
+        if (priv->init_timer) {
+                g_source_remove (priv->init_timer);
+                priv->init_timer = 0;
+        }
+
         stop_scroll_anim (priv);
         if (priv->idle_id) {
                 g_source_remove (priv->idle_id);
@@ -921,25 +933,40 @@ start_aggregator (ELHomeApplet *self)
 }
 
 static gchar*
-format_time (time_t t)
+format_time (time_t t, gboolean time_fmt_24h)
 {
-        static const guint RESULT_SIZE = 32;
+        static const guint RESULT_SIZE = 64;
 
         time_t now;
         struct tm now_tm, t_tm;
-        const gchar *format = "%x %X";
+        const gchar *time_format;
         gchar *result = g_malloc0 (RESULT_SIZE);
 
         now = time (NULL);
         localtime_r (&now, &now_tm);
         localtime_r (&t, &t_tm);
 
+        if (time_fmt_24h)
+                time_format = "wdgt_va_24h_time";
+        else
+                time_format = now_tm.tm_hour > 11 ?
+                        "wdgt_va_12h_time_pm":
+                        "wdgt_va_12h_time_am";
+
         if ((now_tm.tm_year == t_tm.tm_year) &&
             (now_tm.tm_mon  == t_tm.tm_mon) &&
             (now_tm.tm_mday == t_tm.tm_mday))
-                format = "%X";
-
-        strftime (result, RESULT_SIZE, format, &t_tm);
+                strftime (result,
+                          RESULT_SIZE,
+                          dgettext ("hildon-libs", time_format),
+                          &t_tm);
+        else {
+                gchar *full_format = g_strdup_printf ("%s %s",
+                                                      dgettext ("hildon-libs", "wdgt_va_date"),
+                                                      dgettext ("hildon-libs", time_format));
+                strftime (result, RESULT_SIZE, full_format, &t_tm);
+                g_free (full_format);
+        }
 
         return result;
 }
@@ -970,7 +997,7 @@ show_event (ELHomeApplet *self, RTComElIter *it)
 
                         priv->event_id = g_value_get_int (g_value_array_get_nth (event, 0));
                         received_t = g_value_get_int (g_value_array_get_nth (event, 1));
-                        received = format_time (received_t);
+                        received = format_time (received_t, priv->time_fmt_24h);
                         priv->local_id = _VARR_DUP_STR (event, 2);
                         priv->remote_id = _VARR_DUP_STR (event, 3);
                         if (priv->remote_id && priv->remote_id[0]) {
@@ -1192,11 +1219,13 @@ read_event (ELHomeApplet *self)
         if (priv->event_id >= 0) {
                 gboolean new_account = g_strcmp0 (priv->local_id, local_id);
 
-                if (g_strcmp0 (priv->remote_id, remote_id) || new_account) {
+                if (g_strcmp0 (priv->remote_id, remote_id) ||
+                    new_account ||
+                    !priv->contact) {
                         clean_contact (self);
                         start_aggregator (self);
                 }
-                else {
+                else if (priv->contact) {
                         show_contact (self);
                 }
 
@@ -1442,7 +1471,12 @@ read_new_event (ELHomeApplet *self)
         ELHomeAppletPrivate *priv = self->priv;
 
         read_event (self);
-        priv->unread_count = query_unread_events (priv->eventlogger);
+
+        if (priv->event_id >= 0)
+                priv->unread_count = query_unread_events (priv->eventlogger);
+        else
+                priv->unread_count = 0;
+
         update_unread_label (self);
 
         priv->idle_id = 0;
@@ -1485,6 +1519,28 @@ new_event_cb (RTComEl      *backend,
                 add_new_idle (self);
 }
 
+static void
+all_deleted_cb (RTComEl      *backend,
+                const gchar  *service,
+                ELHomeApplet *self)
+{
+        new_event_cb (backend,
+                      0,
+                      NULL,
+                      NULL,
+                      NULL,
+                      NULL,
+                      service,
+                      self);
+}
+
+static void
+refresh_hint_cb (RTComEl     *backend,
+                 ELHomeApplet *self)
+{
+        add_new_idle (self);
+}
+
 static gboolean
 scroll_anim_cb (ELHomeApplet *self)
 {
@@ -1632,9 +1688,7 @@ button_release_event_cb (GtkWidget      *widget,
                                                               self);
                 }
                 else
-#ifndef DEBUG_LAYOUT
                         mark_as_read (self);
-#endif
                 break;
         default:;
         }
@@ -1667,6 +1721,67 @@ leave_notify_event_cb (GtkWidget        *widget,
         return FALSE;
 }
 
+static gboolean
+init_eventlogger (ELHomeApplet *self)
+{
+        ELHomeAppletPrivate *priv = self->priv;
+
+        priv->eventlogger = rtcom_el_new ();
+
+        /* check that db is initialized */
+        gpointer *db = NULL;
+        g_object_get (priv->eventlogger, "db", &db, NULL);
+        if (!db) {
+                static int trial = 0;
+
+                g_object_unref (priv->eventlogger);
+                priv->eventlogger = NULL;
+
+                if (trial == 0) {
+                        trial++;
+                        priv->init_timer = g_timeout_add_seconds (5,
+                                                                  (GSourceFunc)init_eventlogger,
+                                                                  self);
+                        return TRUE; /* return value doesn't matter */
+                }
+                else if (trial < 5) {
+                        trial++;
+                        return TRUE;
+                }
+                else {
+                        g_error ("Failed to init eventlogger");
+                        return FALSE;
+                }
+        }
+
+        g_signal_connect (priv->eventlogger,
+                          "new-event",
+                          G_CALLBACK (new_event_cb),
+                          self);
+        g_signal_connect (priv->eventlogger,
+                          "event-updated",
+                          G_CALLBACK (new_event_cb),
+                          self);
+        g_signal_connect (priv->eventlogger,
+                          "event-deleted",
+                          G_CALLBACK (new_event_cb),
+                          self);
+        g_signal_connect (priv->eventlogger,
+                          "all-deleted",
+                          G_CALLBACK (all_deleted_cb),
+                          self);
+        g_signal_connect (priv->eventlogger,
+                          "refresh-hint",
+                          G_CALLBACK (refresh_hint_cb),
+                          self);
+
+        add_new_idle (self);
+
+        priv->init_timer = 0;
+
+        return FALSE;
+}
+
 static void
 el_home_applet_init (ELHomeApplet *self)
 {
@@ -1674,13 +1789,14 @@ el_home_applet_init (ELHomeApplet *self)
         GtkWidget *event_box;
         GtkWidget *hbox, *vbox, *align, *footer;
         GtkWidget *w;
+        GConfClient *gconf;
 
         self->priv = EL_HOME_APPLET_GET_PRIVATE (self);
         priv = self->priv;
 
         gtk_widget_set_app_paintable (GTK_WIDGET (self), TRUE);
 
-        priv->unread = gtk_label_new ("12");
+        priv->unread = gtk_label_new (NULL);
         gtk_misc_set_alignment (GTK_MISC (priv->unread),
                                 0.0f,
                                 0.5f);
@@ -1691,7 +1807,7 @@ el_home_applet_init (ELHomeApplet *self)
                                 0.5f,
                                 0.5f);
 
-        priv->sender = gtk_label_new ("asdf asdf asdf asdf asdf");
+        priv->sender = gtk_label_new (NULL);
         gtk_misc_set_alignment (GTK_MISC (priv->sender),
                                 0.5f,
                                 0.55f);
@@ -1703,8 +1819,6 @@ el_home_applet_init (ELHomeApplet *self)
                                      C_WIDTH,
                                      HILDON_ICON_PIXEL_SIZE_THUMB);
 
-        priv->message = g_strdup ("One two three four five six seven eight nine ten");
-
         /* construt empty table */
         priv->empty = gtk_fixed_new ();
 
@@ -1734,7 +1848,7 @@ el_home_applet_init (ELHomeApplet *self)
         gtk_widget_hide (GTK_WIDGET (priv->empty));
         GTK_WIDGET_SET_FLAGS (priv->empty, GTK_NO_SHOW_ALL);
 
-        priv->received = gtk_label_new ("aewf aewf aewf awef");
+        priv->received = gtk_label_new (NULL);
         gtk_misc_set_alignment (GTK_MISC (priv->received),
                                 1.0f,
                                 0.5f);
@@ -1802,21 +1916,16 @@ el_home_applet_init (ELHomeApplet *self)
 
         gtk_widget_show_all (GTK_WIDGET (event_box));
 
-#ifndef DEBUG_LAYOUT
-        priv->eventlogger = rtcom_el_new ();
-        g_signal_connect (priv->eventlogger,
-                          "new-event",
-                          G_CALLBACK (new_event_cb),
-                          self);
-        g_signal_connect (priv->eventlogger,
-                          "event-updated",
-                          G_CALLBACK (new_event_cb),
-                          self);
 
         osso_abook_init_with_name (PACKAGE, NULL);
 
-        add_new_idle (self);
-#endif
+        gconf = gconf_client_get_default ();
+        priv->time_fmt_24h = gconf_client_get_bool (gconf,
+                                                    "/apps/clock/time-format",
+                                                    NULL);
+        g_object_unref (gconf);
+
+        init_eventlogger (self);
 }
 
 static void
@@ -1832,4 +1941,3 @@ el_home_applet_class_init (ELHomeAppletClass *klass)
 
         g_type_class_add_private (klass, sizeof (ELHomeAppletPrivate));
 }
-
